// Copyright (c) 2012 by István Váradi

// This file is part of libxplcommon, a common utility library for
// development related to X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "WaitableEvent.h"

#include <cassert>
#include <cerrno>
#include <cstdio>

#include <poll.h>
#include <unistd.h>

#include <sys/eventfd.h>

//------------------------------------------------------------------------------

using xplcommon::posix::WaitableEvent;

//------------------------------------------------------------------------------

WaitableEvent::WaitableEvent(Waiter* waiter) :
    Waitable(waiter, eventfd(0, EFD_NONBLOCK), POLLIN)
{
    assert(fd>=0);
}

//------------------------------------------------------------------------------

WaitableEvent::~WaitableEvent()
{
    close(fd);
}

//------------------------------------------------------------------------------

bool WaitableEvent::ready()
{
    return fired;
}

//------------------------------------------------------------------------------

void WaitableEvent::fire()
{
    static const uint64_t one = 1;
    if (write(fd, &one, sizeof(one))<0) {
        if (errno!=EAGAIN) {
            perror("xplcommon::posix::WaitableEvent::fire: write");
            assert(0);
        }
    }
}

//------------------------------------------------------------------------------

bool WaitableEvent::check()
{
    if (fired) {
        fired = false;
        return true;
    } else {
        return false;
    }
}

//------------------------------------------------------------------------------

void WaitableEvent::handleEvents(short events)
{
    if ((events&POLLIN)==POLLIN) {
        uint64_t value = 0;
        if (read(fd, &value, sizeof(value))<0) {
            if (errno!=EAGAIN) {
                perror("xplcommon::posix::WaitableEvent::handleEvents: read");
                assert(0);
            }
        } else {
            fired = value>0;
        }
    }
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
